IMAGE_INFOS					STRUCT
	hImage					HBITMAP			?
	Image					BITMAP			<>
	ImageInfo				BITMAPINFO		<>
	ImageSens				DWord			?			; Sens de l'image
	dwImageRatio			REAL10			?
	dwImageData				DWord			?
	lpImageBits				LPBYTE			?
IMAGE_INFOS					ENDS

LPIMAGE_INFOS	TYPEDEF	PTR IMAGE_INFOS

;	========================================================================================================================
							.Code
;	========================================================================================================================

FreeImageFile			PROC	USES EBX EDI ESI,lpImage:Ptr IMAGE_INFOS

						mov		edi,lpImage

						test	edi,edi
						jz		@Error

							cmp		(IMAGE_INFOS Ptr [edi]).hImage,NULL
							je		@F

								INVOKE	DeleteObject,(IMAGE_INFOS Ptr [edi]).hImage

@@ :

							cmp		(IMAGE_INFOS Ptr [edi]).lpImageBits,NULL
							je		@Error

								INVOKE	Memory_Free,(IMAGE_INFOS Ptr [edi]).lpImageBits

								push	edi
								push	OFFSET @End
								jmp		Memory_Free

@Error :

						xor		eax,eax
						jmp		@Exit

@End :

						mov		eax,1

@Exit :

;	===========================
;	=== Fin de la procdure ===
;	===========================

						ret
FreeImageFile			ENDP

;	========================================================================================================================

LoadImageFromFile		PROC	USES EBX EDI ESI,lpszFileName:LPSTR

						INVOKE	Memory_Alloc,SIZEOF IMAGE_INFOS

						test	eax,eax
						jz		@Error

						mov		edi,eax

;	========================================
;	=== Charger l'image depuis le disque ===
;	========================================

						INVOKE	BitmapFromFile,lpszFileName	; Charger l'image

						test	eax,eax						; Cannot load bitmap
						jz		@Error

						mov		DWord Ptr (IMAGE_INFOS Ptr [edi]).hImage,eax		; Bitmap handle in EAX

;	============================================================
;	=== Obtenir les informations relatives  l'image charge ===
;	============================================================

						lea		edx,(IMAGE_INFOS Ptr [edi]).Image

						push	edx							; Bitmap structure
						push	SIZEOF BITMAP				; Bitmap structure size
						push	eax							; Bitmap handle
						call	GetObject 

;	=============================================
;	=== Initialize BITMAPINFOHEADER structure ===
;	=============================================

						mov		DWord Ptr (IMAGE_INFOS Ptr [edi]).ImageInfo.bmiHeader.biSize,SIZEOF BITMAPINFOHEADER

						mov		eax,DWord Ptr (IMAGE_INFOS Ptr [edi]).Image.bmWidth
						mov		DWord Ptr (IMAGE_INFOS Ptr [edi]).ImageInfo.bmiHeader.biWidth,eax

						mov		eax,DWord Ptr (IMAGE_INFOS Ptr [edi]).Image.bmHeight
						mov		DWord Ptr (IMAGE_INFOS Ptr [edi]).ImageInfo.bmiHeader.biHeight,eax

						mov		DWord Ptr (IMAGE_INFOS Ptr [edi]).ImageInfo.bmiHeader.biPlanes,1

						mov		ax,Word Ptr (IMAGE_INFOS Ptr [edi]).Image.bmBitsPixel
						mov		Word Ptr (IMAGE_INFOS Ptr [edi]).ImageInfo.bmiHeader.biBitCount,ax

						mov		DWord Ptr (IMAGE_INFOS Ptr [edi]).ImageInfo.bmiHeader.biCompression,BI_RGB

;	=========================================================================
;	=== compute W/H or H/W for later use. If you want to resize the image ===
;	=========================================================================

						mov		eax,DWord Ptr (IMAGE_INFOS Ptr [edi]).Image.bmWidth

						cmp		eax,DWord Ptr (IMAGE_INFOS Ptr [edi]).ImageInfo.bmiHeader.biHeight
						jg		@Horizontale

						fild	DWord Ptr (IMAGE_INFOS Ptr [edi]).Image.bmWidth
						fidiv	DWord Ptr (IMAGE_INFOS Ptr [edi]).ImageInfo.bmiHeader.biHeight
						fst		QWord Ptr (IMAGE_INFOS Ptr [edi]).dwImageRatio

						mov		(IMAGE_INFOS Ptr [edi]).ImageSens,IMAGE_VERTICALE
						jmp		@Continue

@Horizontale :

						fild	DWord Ptr (IMAGE_INFOS Ptr [edi]).ImageInfo.bmiHeader.biHeight
						fidiv	DWord Ptr (IMAGE_INFOS Ptr [edi]).Image.bmWidth
						fst		QWord Ptr (IMAGE_INFOS Ptr [edi]).dwImageRatio

						mov		(IMAGE_INFOS Ptr [edi]).ImageSens,IMAGE_HORIZONTALE

@Continue :

;	=====================
;	=== Create one DC ===
;	=====================

						push	NULL
						call	GetDC

						test	eax,eax
						jz		@Error

						mov		esi,eax						; ESI = hDC

;	=======================
;	=== Get image infos ===
;	=======================

						push	DIB_RGB_COLORS
						lea		eax,(IMAGE_INFOS Ptr [edi]).ImageInfo
						push	eax
						push	NULL
						push	DWord Ptr (IMAGE_INFOS Ptr [edi]).ImageInfo.bmiHeader.biHeight
						push	0
						push	DWord Ptr (IMAGE_INFOS Ptr [edi]).hImage
						push	esi
						call	GetDIBits

;	=============================
;	=== Allocate memory block ===
;	=============================

						INVOKE	Memory_Alloc,(IMAGE_INFOS Ptr [edi]).ImageInfo.bmiHeader.biSizeImage

						mov		(IMAGE_INFOS Ptr [edi]).lpImageBits,eax

						test	eax,eax
						jz		@Error

;	======================
;	=== Get image bits ===
;	======================

						push	DIB_RGB_COLORS										; RGB or palette index
						lea		eax,(IMAGE_INFOS Ptr [edi]).ImageInfo
						push	eax													; bitmap data buffer
						push	(IMAGE_INFOS Ptr [edi]).lpImageBits									; array for bitmap bits
						push	DWord Ptr (IMAGE_INFOS Ptr [edi]).ImageInfo.bmiHeader.biHeight		; number of scan lines to copy
						push	0													; first scan line to set
						push	DWord Ptr (IMAGE_INFOS Ptr [edi]).hImage								; handle to bitmap
						push	esi													; handle to DC
						call	GetDIBits

						push	eax

;	====================
;	=== Frees the DC ===
;	====================

						push	esi
						push	NULL
						call	ReleaseDC

						pop		eax

						test	eax,eax
						jnz		@Finished

;	========================================================
;	=== Frees memory because some operations have failed ===
;	========================================================

@Error :

						cmp		(IMAGE_INFOS Ptr [edi]).hImage,NULL
						je		@Error_1

							INVOKE	DeleteObject,(IMAGE_INFOS Ptr [edi]).hImage

@Error_1 :

						INVOKE	Memory_Free,(IMAGE_INFOS Ptr [edi]).lpImageBits
						INVOKE	Memory_Free,edi

						xor		eax,eax						; Operations have failed
						jmp		@Exit

@Finished :

						mov		eax,edi

@Exit :

;	===========================
;	=== Fin de la procdure ===
;	===========================

						ret
LoadImageFromFile		ENDP

;	========================================================================================================================
